<?php

/**
 * Class JobsImporter - Import data to WP posts with custom fields
 *
 * @uses    Pods framework
 * @package Biuro Jobs importer
 * @used-by \jobs-importer\job-importer.php
 */

class JobsImporter
{

    private $data;
    private $log;

    // Biuro record unique key
    protected $primaryKey = 'livas-id';

    // Language attribute key
    protected $lang_id = 'lang';

    // Pods instance
    protected $pods;

    // Posts container in WP
    protected $podsType = 'job';

    // Additional post fields
    protected $postProperties = [
        'post_status'    => 'publish',
        'comment_status' => 'closed'
    ];

    protected $primaryTerms = [];

    public function __construct($data, $log = true)
    {
        if (!function_exists('is_plugin_active')) {
            require_once 'wp-admin/includes/plugin.php';
        }

        $this->data = $data;
        $this->log = $log;
        $this->setPods();
    }

    /**
     *  Set Pods instance
     */
    private function setPods()
    {
        $this->pods = pods($this->podsType);
    }

    /**
     *  Transform data keys by relations mapping
     *
     * @param array $data Data array
     *
     * @param array $relationsMap
     *
     * @return array
     */
    private function mapRelations($data, $relationsMap = [])
    {
        foreach ($relationsMap as $relationKey => $dataKey) {
            if (array_key_exists($dataKey, $data)) {
                $data[$relationKey] = $data[$dataKey];
                unset($data[$dataKey]);
            }
        }

        return $data;
    }

    public static function importPositions () {
        $env = getEnvironment();

        $inputFile = getSource($env);

        $ads = (new JsonDataCollector($inputFile))->getData();

        (new JobsImporter($ads, false))->import();
        exit;
    }

    /**
     *  Import posts from provided data: if post with given Id exists, it rewrites
     */
    public function import()
    {
        global $permalink_manager_uris;

        foreach ($this->data as $ad) {

            $this->primaryTerms = [];

            // Set language code
            $lang = $this->getAdLang($ad);
            // Set relations mapping
            $relations = $ad['relations'];
            // Set related taxonomies
            $terms = $this->getTerms($ad['terms'], $lang);
            // Set related custom posts
            $posts = $this->getPosts($ad['posts'], $lang);

            // Collect and prepare job's data
            $adData = $this->mapRelations(array_merge(
                $ad['properties'],
                ['lang' => $lang],
                $terms,
                $posts,
                $this->postProperties
            ),
                $relations
            );

            // Unique key of post
            $primaryKey = $adData[$this->primaryKey];

            $pod = $this->getPod($primaryKey);

            if ($pod->total_found() > 0) {

                try {

                    $podId = $pod->save($adData);
                    // Set post language
                    $this->setPostLang($podId, $ad['lang']);
                    if ($this->log) {
                        print_r("Job ID: <b>$podId</b> <br>");
                        print_r("Old record ($this->primaryKey: $primaryKey) has been updated. <br>");
                    }

                } catch (Exception $e) {
                    if ($this->log) {
                        print_r("Error while updating record ($this->idKey: $primaryKey). Error: $e->getMessage()<br>");
                    }
                }

            } else {

                try {

                    $podId = $this->pods->add($adData);
                    // Set post language
                    $this->setPostLang($podId, $ad['lang']);
                    $this->getPod($primaryKey)->save($adData); // Save related data, cause didn't saved on creation
                    if ($this->log) {
                        print_r("New record ($this->primaryKey: $primaryKey) has been created. <br>");
                    }
                } catch (Exception $e) {
                    if ($this->log) {
                        print_r("Error while creating record ($this->primaryKey: $primaryKey). Error: $e->getMessage()<br>");
                    }
                }
            }

            // Set primary properties
            if (is_plugin_active('wordpress-seo/wp-seo.php')) {

                foreach ($this->primaryTerms as $term => $termKey) {

                    update_post_meta($podId, "_yoast_wpseo_primary_{$term}", $termKey);
                }
            }

            $this->addSeoProperties($ad, $podId);
        }
    }

    /**
     *  Add SEO properties for ad
     * @uses Yoast SEO plugin
     *
     * @param $ad
     * @param $postId
     */
    private function addSeoProperties($ad, $postId)
    {
        if (is_plugin_active('wordpress-seo/wp-seo.php')) {

            foreach ($ad['seo_properties'] as $key => $item) {

                update_post_meta($postId, '_yoast_wpseo_' . $key, $item);
            }
        }
    }

    /**
     *  Get taxonomies Id's
     *
     * @param array $terms Data array
     * @param string $lang Language code
     *
     * @return array
     */
    private function getTerms($terms, $lang)
    {
        $result = [];

        foreach ($terms as $termKey => $termsData) {
            foreach ($termsData as $key => $data) {

                $primTerm = false;
                if (isset($data['__primary'])) {
                    $primTerm = true;
                    unset($data['__primary']);
                }

                $data['slug'] = $this->getSlug($termKey, $data, $lang);
                $term = $this->getTerm($termKey, $data, $lang);
                $result[$termKey][] = $term;

                if ($primTerm) {
                    $this->primaryTerms[$termKey] = $term;
                }
            }
        }

        return $result;
    }

    /**
     *  Get custom posts Id's
     *
     * @param $posts array Data array
     * @param $lang string Language code
     *
     * @return array
     */
    private function getPosts($posts, $lang)
    {
        $result = [];

        foreach ($posts as $postKey => $postData) {
            $result[$postKey] = $this->getPost($postKey, $postData, $lang);
        }

        return $result;
    }

    /**
     *  Return formated slug
     *
     * @param string $key Object key name
     * @param array $data Object data
     * @param string $lang Language code
     *
     * @return string Slug
     */
    private function getSlug($key, $data, $lang)
    {
        if (isset($data['slug'])) {
            return $key . '-' . $data['slug'] . '-' . $lang;
        }

        return $data['name'] . '-' . $lang;
    }

    /**
     *  Find and return taxonomy id, if not exist, create it
     *
     * @param string $key Taxonomy key name
     * @param $data
     * @param string $lang Taxonomy language code
     *
     * @return int Taxonomy id
     */
    private function getTerm($key, $data, $lang)
    {
        $term = pods($key, $data['slug']);

        if (!$term->exists()) {

            try {

                $termId = pods($key)->add($data);

            } catch (Exception $e) {
                if ($this->log) {
                    print_r("Error while creating taxonomies ($key => $data[slug]). Error: $e->getMessage()<br>");
                }
            }
        } else {

            try {

                $termId = $term->save($data);

            } catch (Exception $e) {
                if ($this->log) {
                    print_r("Error while updating taxonomies ($key => $data[slug]). Error: $e->getMessage()<br>");
                }
            }

        }

        if (empty($termId)) {

            return false;
        }

        $this->setTermLang($termId, $lang);

        return $termId;
    }

    /**
     *  Get custom post Id, if post not exists, creates it
     *
     * @param string $key Post key name
     * @param array $data Post data array
     * @param string $lang Post language code
     *
     * @return int Post Id
     */
    private function getPost($key, $data, $lang)
    {
        $slug = $this->getSlug($key, $data, $lang);
        $post = pods($key, $slug);

        if (!$post->exists()) {

            try {

                $postId = pods($key)->add(
                    array_merge(
                        $data,
                        ['slug' => $slug],
                        $this->postProperties
                    )
                );

            } catch (Exception $e) {
                if ($this->log) {
                    print_r("Error while creating custom post ($key => $slug). Error: $e->getMessage()<br>");
                }
            }
        } else {

            try {

                $postId = $post->save(
                    array_merge(
                        $data,
                        ['slug' => $slug],
                        $this->postProperties
                    )
                );

            } catch (Exception $e) {
                if ($this->log) {
                    print_r("Error while updating custom post ($key => $slug). Error: $e->getMessage()<br>");
                }
            }
        }

        if (empty($postId)) {

            return false;
        }

        $this->setPostLang($postId, $lang);

        return $postId;

    }

    /**
     *  Set language param for given pod, using Polylang library
     *
     * @param $postId
     * @param $lang
     */
    private function setPostLang($postId, $lang)
    {
        try {

            PLL()->model->post->set_language($postId, $lang);
            if ($this->log) {
                print_r("Set language code '$lang' for custom post (id: $postId). <br>");
            }

        } catch (Exception $e) {
            if ($this->log) {
                print_r("Error while setting language code for custom post (id: $postId), error: $e->getMessage().<br>");
            }
        }
    }

    /**
     *  Set language param for given taxonomy, using Polylang library
     *
     * @param $termId
     * @param $lang
     */
    private function setTermLang($termId, $lang)
    {
        try {

            PLL()->model->term->set_language($termId, $lang);
            if ($this->log) {
                print_r("Set language code '$lang' for taxonomy (id: $termId). <br>");
            }

        } catch (Exception $e) {
            if ($this->log) {
                print_r("Error while setting language code for taxonomy (id: $termId), error: $e->getMessage().<br>");
            }
        }
    }

    /**
     *  Get pod by given id
     *
     * @param $id
     *
     * @return mixed
     */
    private function getPod($id)
    {
        $params = [
            'where' => [
                'relation' => 'AND',
                [
                    'value'   => (int)$id,
                    'key'     => $this->primaryKey,
                    'compare' => '=',
                ]
            ]
        ];

        return $this->pods->find($params);
    }

    /**
     *  Get language code from advert data
     *
     * @param array $ad
     *
     * @return string`
     */
    private function getAdLang($ad)
    {
        return $ad[$this->lang_id];
    }


    private function add_to_yoast_seo($post_id, $metatitle, $metadesc, $metakeywords){

        $ret = false;

        // Include plugin library to check if Yoast Seo is presently active
        include_once( ABSPATH.'panel/includes/plugin.php' );

        if(is_plugin_active('wordpress-seo/wp-seo.php')) {
            //plugin is activated
            $updated_title = update_post_meta($post_id, '_yoast_wpseo_title', $metatitle);
            $updated_desc = update_post_meta($post_id, '_yoast_wpseo_metadesc', $metadesc);
            $updated_kw = update_post_meta($post_id, '_yoast_wpseo_metakeywords', $metakeywords);

            if($updated_title && $updated_desc && $updated_kw){
                $ret = true;
            }
        }

        return $ret;
    }

}
